#include <stdio.h>
#include <stdlib.h>

#include "tree.h"

int tree_insert(Tree *tree, int data) {
  Node *new_node = malloc(sizeof(Node));
  new_node->data = data;
  new_node->left = NULL;
  new_node->right = NULL;

  // If the tree is empty
  if (tree->root == NULL) {
    tree->root = new_node;
    return 0; // Early return
  }

  // If the tree is non-empty
  Node *cursor = tree->root;

  // An iterative (non-recursive approach) to tree insertion
  // While we look for a place to put our new node
  while (1) {
    // If larger than data
    if (new_node->data > cursor->data) {
      // If there is a child to the right
      if (cursor->right != NULL)
        cursor = cursor->right;
      else {
        cursor->right = new_node; // Put our node here
        break;                    // Break the outer while loop
      }
    }

    // If less-or-equal than our data
    if (new_node->data <= cursor->data) {
      // If there is a child to the left
      if (cursor->left != NULL)
        cursor = cursor->left;
      else {
        cursor->left = new_node; // Put our node here
        break;                   // Break the outer while loop
      }
    }
  }

  return 0;
}

bool tree_contains(Node *node, int data) {
  if (node == NULL)
    return false;

  if (node->data == data)
    return true;

  return tree_contains(node->left, data) || tree_contains(node->right, data);
}

int tree_remove(Node **node, int data) {
  if (*node == NULL)
    return 0;

  if ((*node)->data == data) {
    // If the node has no children
    if ((*node)->left == NULL && (*node)->right == NULL) {
      free(*node);
      *node = NULL;
      return 0;
    }

    // If the node has one child
    if ((*node)->left == NULL) {
      Node *temp = *node;
      *node = (*node)->right;
      free(temp);
      return 0;
    }

    if ((*node)->right == NULL) {
      Node *temp = *node;
      *node = (*node)->left;
      free(temp);
      return 0;
    }

    // If the node has two children
    Node *temp = (*node)->right;
    while (temp->left != NULL)
      temp = temp->left;

    (*node)->data = temp->data;
    return tree_remove(&(*node)->right, temp->data);
  }

  if ((*node)->data < data)
    return tree_remove(&(*node)->right, data);

  return tree_remove(&(*node)->left, data);
}

void tree_clear(Tree *tree) {
  if (tree->root == NULL)
    return;

  node_clear(tree->root);
  tree->root = NULL;
}

void node_clear(Node *node) {
  if (node == NULL)
    return;

  if (node->left != NULL)
    node_clear(node->left);

  if (node->right != NULL)
    node_clear(node->right);

  free(node);
}

void tree_inorder(Node *node) {
  if (node == NULL)
    return;

  tree_inorder(node->left);
  printf("%d ", node->data);
  tree_inorder(node->right);
}

int tree_size(Node *node) {
  if (node == NULL) {
    return 0;
  }

  return 1 + tree_size(node->left) + tree_size(node->right);
}
