#ifndef TREE_H
#define TREE_H

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>

/**
 * @brief A node in a binary tree
 * @details A node in a binary tree has some data
 *          and can have a left and right branch
 */
struct Node {
  /** @brief The data of the node */
  int data;

   /** @see Node */
  struct Node *left;

  /** @see Node */
  struct Node *right;
};
typedef struct Node Node;

/** @brief Represents a tree with a root node */
struct Tree {
  /** @brief The root node of the tree */
  Node *root;
};
typedef struct Tree Tree;

/**
 * @brief Insert some data into the tree
 * 
 * @param tree 
 * @param data 
 * @return * Insert 
 */
int tree_insert(Tree *tree, int data);

// Remove some data from the tree
int tree_remove(Node **node, int data);

// Clear the tree (this wraps node_clear)
void tree_clear(Tree *tree);

// Recursively clear from a certain node
void node_clear(Node *node);

// Get the size of the tree
int tree_size(Node *node);

// Check if the tree contains some data
bool tree_contains(Node *node, int data);

// Print the tree in order
void tree_inorder(Node *node);

#endif // TREE_H